#include <qapplication.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpainter.h>
#include <iostream>
#include <QTimer>
#include <qwt_plot_layout.h>
#include <qwt_plot_grid.h>
#include <qwt_plot_zoomer.h>
#include <qwt_slider.h>
#include <qwt_plot_curve.h>
#include <qwt_scale_draw.h>
#include <qwt_scale_widget.h>
#include <qwt_legend.h>
#include <qwt_legend_item.h>

#include <qdatetime.h>
#include "freqplot.h"
#include "temp.h"

class TimeScaleDraw: public QwtScaleDraw
{
public:
    TimeScaleDraw(const QTime &base):
        baseTime(base)
    {
    }
    virtual QwtText label(double v) const
    {
        QTime upTime = baseTime.addSecs((int)v);
        return upTime.toString();
    }
private:
    QTime baseTime;
};

class Background: public QwtPlotItem
{
public:
    Background()
    {
        setZ(0.0);
    }

    virtual int rtti() const
    {
        return QwtPlotItem::Rtti_PlotUserItem;
    }

    virtual void draw(QPainter *painter,
        const QwtScaleMap &, const QwtScaleMap &yMap,
        const QRect &rect) const
    {
        QColor c(Qt::white);
        QRect r = rect;

        /*for ( int i = 100; i > 0; i -= 10 )
        {
            r.setBottom(yMap.transform(i - 10));
            r.setTop(yMap.transform(i));
            painter->fillRect(r, c);

            c = c.dark(110);
        }
		*/
	
    }
};

class FreqCurve: public QwtPlotCurve
{
public:
    FreqCurve(const QString &title):
        QwtPlotCurve(title)
    {
        //setRenderHint(QwtPlotItem::RenderAntialiased);
    }

    void setColor(const QColor &color)
    {
        QColor c = color;
        c.setAlpha(150);

        setPen(c);
        //setBrush(c);
    }
};

FreqPlot::FreqPlot( int sensor, int type):
    dataCount(0)
{
	/*QwtSlider *slider = NULL;
	slider = new QwtSlider(parent,Qt::Vertical, QwtSlider::NoScale, QwtSlider::BgTrough);
	slider->setRange(0.0,100.0,1.0, 10);*/
	setAutoReplot(true);
	sensorX = sensor;
	typeX = type;

	plotLayout()->setAlignCanvasToScales(true);

	QwtLegend *legend = new QwtLegend;
	legend->setItemMode(QwtLegend::CheckableItem);
	insertLegend(legend, QwtPlot::RightLegend);

	setAxisScaleDraw(QwtPlot::xBottom,  new TimeScaleDraw(upTime()));
	setAxisScale(QwtPlot::xBottom, 0, HISTORY);
	setAxisLabelRotation(QwtPlot::xBottom, -30.0);
	setAxisLabelAlignment(QwtPlot::xBottom, Qt::AlignLeft | Qt::AlignBottom);

	QwtScaleWidget *scaleWidget = axisWidget(QwtPlot::xBottom);
	const int fmh = QFontMetrics(scaleWidget->font()).height();
	scaleWidget->setMinBorderDist(0, fmh / 2);

	//setAxisScale(QwtPlot::yLeft, 0, 100);

	Background *bg = new Background();
	bg->attach(this);

    
	FreqCurve *curve;
	curve = new FreqCurve("Freq");
	curve->setColor(Qt::blue);
	curve->setStyle(QwtPlotCurve::Lines);
	curve->attach(this);
	data[User].curve = curve;

	showCurve(data[User].curve, true);

	for ( int i = 0; i < HISTORY; i++ )
		timeData[HISTORY - 1 - i] = i;
	
	//(void)startTimer(100); 
	connect(this, SIGNAL(legendChecked(QwtPlotItem *, bool)),SLOT(showCurve(QwtPlotItem *, bool)));
	
	refreshTimer = new QTimer();
	connect(refreshTimer, SIGNAL(timeout()), SLOT(timerEvent()));
	refreshTimer->start(1000);




	QwtPlotGrid *grid = new QwtPlotGrid();
	//enable the x and y axis lines
	grid->enableX(true);
	grid->enableY(true);
	//set the X and Y division and scale to that of the channels
	grid->setXDiv(*(this->axisScaleDiv(QwtPlot::xBottom)));
	grid->setYDiv(*(this->axisScaleDiv(QwtPlot::yLeft)));
	//attach the grid to the plot
	grid->attach(this);


	//create the zooming widget
	//jmfriedt a enleve cette ligne
//	QwtPlotZoomer *zoom = new QwtPlotZoomer(QwtPlot::xBottom,QwtPlot::yLeft,(QwtPicker::CornerToCorner | QwtPicker::RectSelection),QwtPicker::AlwaysOn,this->canvas(),true);
//set the zooming policy to reflect the new data
//		zoom->setEnabled(false);

	this->setMinimumSize(50,50);
}

QTime FreqPlot::upTime() const
{
    QTime t;
	t = t.addSecs(1);

	//t = QTime::currentTime();
    return t;
}


//void FreqPlot::timerEvent(QTimerEvent *)
void FreqPlot::timerEvent()
{
	refreshTimer->setInterval(1000);

	if ( Temp::temperature.size() > 1 )
	{
		for ( int i = dataCount; i > 0; i-- )
		{
			if ( i < HISTORY )
				data[User].data[i] = data[User].data[i-1];
		}

		if ( typeX == 0 )
		{
			
			data[User].data[0] = Temp::temperature[Temp::temperature.size()-1];
			
			setAxisScale(QwtPlot::yLeft, -20, 70 );
			
		QString test = QString(" Temperature (%1)").arg(data[User].data[0]);
		setTitle(test);
    	setAxisTitle(QwtPlot::yLeft, "Temperature");
		}
	
		if ( dataCount < HISTORY )
			dataCount++;

		for ( int j = 0; j < HISTORY; j++ )
			timeData[j]++;


		setAxisScale(QwtPlot::xBottom, timeData[HISTORY - 1], timeData[0]);
		data[User].curve->setRawData(timeData, data[User].data, dataCount);
    

	}
	replot();
	
}

void FreqPlot::showCurve(QwtPlotItem *item, bool on)
{
    item->setVisible(on);
    QWidget *w = legend()->find(item);
    if ( w && w->inherits("QwtLegendItem") )
        ((QwtLegendItem *)w)->setChecked(on);
    
    replot();
}
