#include <QtGui>
#include <QSpinBox>
#include <iostream>
#include <QDateTime>
#include <QDate>
#include <QTime>
#include <QStringList>
#include <QFile>
#include "PhotoAutoCapture.h"
#include "qextserialport.h"
#include "qserialporttools.h"


QextSerialPort port;
QSerialPortTools *spc;

QTime 	PhotoAutoCapture::temps 			= QTime::currentTime();
QDate 	PhotoAutoCapture::m_date 			= QDate::currentDate();

PhotoAutoCapture::PhotoAutoCapture()
{	
    centralWidget = new QWidget;
    setCentralWidget(centralWidget);
		
	port.setPortName(QString(""));

	labelDate = new QLabel ; //(dateDuJour);
	faitLabelDate();

	dateTimer = new QTimer();
	dateTimer->start(1000);	
	connect(dateTimer, SIGNAL(timeout()), SLOT(faitLabelDate()));

	labelDepart = new QLabel(tr("Start time  :"));
	Hdebut = new QSpinBox;
	Hdebut->setRange(0, 23);
	Hdeb = new QLabel(tr("Hour"));
	Mindebut = new QSpinBox;
	Mindebut->setRange(0, 59);
	Mindeb = new QLabel(tr("minute"));

	labelInterval = new QLabel(tr("Time interval:"));
	Hinter = new QSpinBox;
	Hinter->setRange(0, 23);
	Hint = new QLabel(tr("Hour"));
	Mininter = new QSpinBox;
	Mininter->setRange(0, 59);
	Minint = new QLabel(tr("minute"));

	labelNbPhoto = new QLabel(tr("Frames / Day:"));
	NbPhoto = new QSpinBox;

	ConfigButton = new QPushButton(tr("Serial port configuration"));
	ConfigButton->setDefault(true);
	ConfigButton->setMinimumSize(60,30);
	connect(ConfigButton, SIGNAL(clicked()), this, SLOT(configPortSlot()));

	SendButton = new QPushButton(tr("&Send"));
	SendButton->setDefault(true);
	SendButton->setMinimumSize(60,30);
	connect(SendButton, SIGNAL(clicked()), this, SLOT(SendData()));

	ReceiveButton = new QPushButton(tr("&Receive"));
	ReceiveButton->setMinimumSize(60,30);
	ReceiveButton->setDefault(true);
	connect(ReceiveButton, SIGNAL(clicked()), this, SLOT(ReceiveData()));
	
	TempButton = new QPushButton(tr("Get Temperature file"));
	TempButton->setMinimumSize(60,30);
	TempButton->setDefault(true);
	connect(TempButton,SIGNAL(clicked()),this,SLOT(ReceiveTemp()));
	
	buttonBox = new QDialogButtonBox(Qt::Vertical);
	buttonBox->addButton(ConfigButton, QDialogButtonBox::ActionRole);
	buttonBox->addButton(SendButton, QDialogButtonBox::ActionRole);
	buttonBox->addButton(ReceiveButton, QDialogButtonBox::ActionRole);
	buttonBox->addButton(TempButton, QDialogButtonBox::ActionRole);
	
	QHBoxLayout *dateLayout = new QHBoxLayout;
	dateLayout->addWidget(labelDate);

	QGridLayout *paramLayout = new QGridLayout;

 	paramLayout->addLayout(dateLayout,0,1,1,5);
	
	paramLayout->addWidget(labelDepart,1,1);
 	paramLayout->addWidget(Hdebut,1,2);
	paramLayout->addWidget(Hdeb,1,3);
 	paramLayout->addWidget(Mindebut,1,4);
	paramLayout->addWidget(Mindeb,1,5);

	paramLayout->addWidget(labelInterval,2,1);
	paramLayout->addWidget(Hinter,2,2);
	paramLayout->addWidget(Hint,2,3);
	paramLayout->addWidget(Mininter,2,4);
	paramLayout->addWidget(Minint,2,5);

	paramLayout->addWidget(labelNbPhoto,3,1);
	paramLayout->addWidget(NbPhoto,3,2,1,3);
	paramLayout->setHorizontalSpacing(15);
	mainLayout = new QGridLayout;
 	mainLayout->addLayout(paramLayout, 0, 0);
	mainLayout->addWidget(buttonBox, 1,0,1,0,Qt::AlignHCenter);
	centralWidget->setLayout(mainLayout);
	
	setWindowTitle(tr("AutoPhotoCapture"));
}

PhotoAutoCapture::~PhotoAutoCapture()
{
	//delete spc;
}

int PhotoAutoCapture::faitLabelDate()
{
	PhotoAutoCapture::m_date = QDate::currentDate();
	PhotoAutoCapture::temps	= QTime::currentTime();
	QString str;
	str.setNum(GetMois());
	QString dateDuJour = "<font size=+2><b>Date:<b/> "+str;
	str.setNum(GetJour());
	dateDuJour = dateDuJour+"/"+str;
	str.setNum(GetAnnee()+2000);
	dateDuJour = dateDuJour+"/"+str;
	
	str.setNum(GetHeure());
	dateDuJour = dateDuJour+"<b> Hours: </b>"+str;
	str.setNum(GetMinute());
	dateDuJour = dateDuJour+"H "+str;
	str.setNum(GetSeconde());
	dateDuJour = dateDuJour+"Min "+str;
	dateDuJour = dateDuJour+"s</font>";

	labelDate->setText(dateDuJour);
}

void PhotoAutoCapture::configPortSlot()
{
	QextSerialPort defaultConfig;
	defaultConfig.setParity(PAR_NONE);
	defaultConfig.setDataBits(DATA_8);
	defaultConfig.setStopBits(STOP_1);
	defaultConfig.setBaudRate(BAUD9600);
	defaultConfig.setFlowControl(FLOW_OFF);
	configPort(defaultConfig);
}

void PhotoAutoCapture::configPort(QextSerialPort &defConfig)
{
	spc = new QSerialPortTools(defConfig,port);
}


void PhotoAutoCapture::SendData()
{
	if ( !spc->portReady(port) )
		configPortSlot();

	if( spc->connectPort(port, QIODevice::ReadWrite ) )
	{ 
		char ctmp = 'P';
		int i = port.write(&ctmp,1);
		char tmp ;

		tmp = (char) GetAnnee();
		i = port.write(&tmp,1);
		tmp = (char) GetMois();
		i = port.write(&tmp,1);
		tmp = (char) GetJour();
		i = port.write(&tmp,1);
		tmp = (char) GetHeure();
		i = port.write(&tmp,1);
		tmp = (char) GetMinute();
		i = port.write(&tmp,1);
		
		tmp = (char) Hdebut->value();
		i = port.write(&tmp,1);
		tmp = (char) Mindebut->value();
		i = port.write(&tmp,1);
		tmp = (char) Hinter->value();
		i = port.write(&tmp,1);
		tmp = (char) Mininter->value();
		i = port.write(&tmp,1);
		tmp = (char) NbPhoto->value();
		i = port.write(&tmp,1);

		spc->disconnectPort(port);

		/** affiche le resultat*/		
		ReceiveData();
	}
}

void PhotoAutoCapture::ReceiveData()
{
	if ( !spc->portReady(port) )
		configPortSlot();

	if(  spc->connectPort(port,QIODevice::ReadWrite) )
	{
		char tmp = '?';
		bool ok;
		QString msg="";
		char buff[12];
		int i = port.write(&tmp,1);
		i = 0;
		while ( i < 12 ){	
			port.read(&buff[i],1);
			i++;
		}
		spc->disconnectPort(port);

		msg.sprintf("\nLocal hour=%d/%d/%d %d:%d\nalarme=%d:%d\nTime interval=%d:%d\n%d Frames/day\nnext alarm : %d:%d\n",buff[0],buff[1],buff[2],buff[3],buff[4],buff[5],buff[6],buff[7],buff[8],buff[9],buff[10],buff[11]);
    	QMessageBox::information(this, tr("Settings :"),msg);
					
		Hdebut->setValue((int) buff[5]);
		Mindebut->setValue((int) buff[6]);
		Hinter->setValue((int) buff[7]);
		Mininter->setValue((int) buff[8]);
		NbPhoto->setValue((int) buff[9]);
	}
}

void PhotoAutoCapture::ReceiveTemp()
{
	if ( !spc->portReady(port) )
		configPortSlot();
	
	if( spc->connectPort(port,QIODevice::ReadWrite) )
	{
		/* ouverture du fichier de sortie */
		QString fileName = QFileDialog::getSaveFileName(this,tr("Where do you want to save temperature list"),"/",tr("*.*"));

		QFile file(fileName);
		if ( !file.open(QIODevice::WriteOnly) ) 
			return;
	
		QTextStream out(&file);
		/*********************************/

		QString retour = "";
		char tmp = 'T';
		port.write(&tmp,1);


		bool charFF = false;
		while ( !charFF ){	
		    int numBytes = port.bytesAvailable();
		    if(numBytes > 0) {
	        	if(numBytes > 3) numBytes = 3;
		        char buff[3];
		        int i = port.read(buff, numBytes);
	    	    buff[numBytes-1] = '\0';
				int d = 0;
				sscanf(buff,"%x",&d);
				float celcius = (((float) d + 256.)/512. * 1.5 - 0.986)/0.00355; 
				out << retour.sprintf("%s   %f\r\n",buff, celcius);
				retour = buff; 
				if ( retour.contains("FF",Qt::CaseInsensitive) || retour.isEmpty() ){
					charFF = true;
					break;
				}
			}
		}
		file.close();
		spc->disconnectPort(port);
	}
}

void PhotoAutoCapture::criticalMessage()
{
	QMessageBox::StandardButton reply;
	reply = QMessageBox::critical(this, tr("Error"),
                                     "Connection not found!",
                                     QMessageBox::Abort );
}

int PhotoAutoCapture::GetHeure()
{
  return temps.hour();
}

int PhotoAutoCapture::GetMinute()
{
  return temps.minute();
}

int PhotoAutoCapture::GetSeconde()
{
  return temps.second();
}

int PhotoAutoCapture::GetAnnee()
{
  return m_date.year()-2000;
}

int PhotoAutoCapture::GetMois()
{
  return m_date.month();
}

int PhotoAutoCapture::GetJour()
{
  return m_date.day();
}
