#include <QtGui>
#include <QWidget>
#include <QGroupBox>
#include <QSpinBox>
#include <iostream>
#include <QString>
#include <QRegExp>
#include <QStringList>
#include <math.h>
#include <QLineEdit>
#include "qserialporttools.h"

QextSerialPort* portSettings;

QSerialPortTools::QSerialPortTools(QextSerialPort &defaultConfig,QextSerialPort &portConfig)
{
	defaultSettings = defaultConfig;
	portSettings = &portConfig;

	QGroupBox *configGroup = new QGroupBox(tr("Serial Port Setting"));

	QLabel *portLabel = new QLabel(tr("Serial port :"));
	portComboBox = new QComboBox;

	
    scanPortButton = new QPushButton(tr("Scan"));
    scanPortButton->setDefault(true);
	scanPortButton->setMinimumSize(20,20);
    connect(scanPortButton, SIGNAL(clicked()), this, SLOT(testAllPort()));


	QLabel *baudRateLabel = new QLabel(tr("Baud Rate :"));
	baudRateComboBox = new QComboBox;

	QLabel *dataBitsTypeLabel = new QLabel(tr("dataBitsType :"));
	dataBitsTypeComboBox = new QComboBox;

	QLabel *parityTypeLabel = new QLabel(tr("ParityType :"));
	parityTypeComboBox = new QComboBox;
	
	QLabel *stopBitsTypeLabel = new QLabel(tr("StopBitsType :"));
	stopBitsTypeComboBox = new QComboBox;

	QLabel *flowTypeLabel = new QLabel(tr("FlowType :"));
	flowTypeComboBox = new QComboBox;

	
    setDefaultButton = new QPushButton(tr("Default"));
    setDefaultButton->setDefault(true);
	setDefaultButton->setMinimumSize(20,20);
    connect(setDefaultButton, SIGNAL(clicked()), this, SLOT(setDefaultSettings()));

    applyButton = new QPushButton(tr("Apply"));
    applyButton->setDefault(true);
	applyButton->setMinimumSize(20,20);
    connect(applyButton, SIGNAL(clicked()), this, SLOT(applySettings()));

	
    cancelButton = new QPushButton(tr("Cancel"));
    cancelButton->setDefault(true);
	cancelButton->setMinimumSize(20,20);
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));

    okButton = new QPushButton(tr("OK"));
    okButton->setDefault(true);
	okButton->setMinimumSize(20,20);
    connect(okButton, SIGNAL(clicked()), this, SLOT(ok()));

	QGridLayout *gridLayout = new QGridLayout;
	gridLayout->addWidget(portLabel,0,0);
	gridLayout->addWidget(portComboBox,0,1,1,2);
	gridLayout->addWidget(scanPortButton,0,3);
	gridLayout->addWidget(baudRateLabel,1,0);
	gridLayout->addWidget(baudRateComboBox,1,1,1,3);
	gridLayout->addWidget(dataBitsTypeLabel,2,0);
	gridLayout->addWidget(dataBitsTypeComboBox,2,1,1,3);
	gridLayout->addWidget(parityTypeLabel,3,0);
	gridLayout->addWidget(parityTypeComboBox,3,1,1,3);
	gridLayout->addWidget(stopBitsTypeLabel,4,0);
	gridLayout->addWidget(stopBitsTypeComboBox,4,1,1,3);
	gridLayout->addWidget(flowTypeLabel,5,0);
	gridLayout->addWidget(flowTypeComboBox,5,1,1,3);
	gridLayout->addWidget(okButton,6,0);
	gridLayout->addWidget(setDefaultButton,6,1);
	gridLayout->addWidget(applyButton,6,2);
	gridLayout->addWidget(cancelButton,6,3);

	setLayout(gridLayout);
	testAllPort();
	completeData();
	setDefaultSettings();

	show();
}

QSerialPortTools::~QSerialPortTools()
{
	//nop
}

void QSerialPortTools::completeData()
{
	baudrate.append(qMakePair( QString("BAUD50                //POSIX ONLY"),BAUD50 ));
	baudrate.append(qMakePair( QString("BAUD75                //POSIX ONLY"),BAUD75 ));
	baudrate.append(qMakePair( QString("BAUD110"), BAUD110));
	baudrate.append(qMakePair( QString("BAUD134               //POSIX ONLY"),BAUD134 ));
	baudrate.append(qMakePair( QString("BAUD150               //POSIX ONLY"),BAUD150  ));
	baudrate.append(qMakePair( QString("BAUD200               //POSIX ONLY"),BAUD200 ));
	baudrate.append(qMakePair( QString("BAUD300"),BAUD300 ));
	baudrate.append(qMakePair( QString("BAUD600"),BAUD600 ));
	baudrate.append(qMakePair( QString("BAUD1200"),BAUD1200 ));
	baudrate.append(qMakePair( QString("BAUD1800              //POSIX ONLY"),BAUD1800 ));
	baudrate.append(qMakePair( QString("BAUD2400"),BAUD2400 ));
	baudrate.append(qMakePair( QString("BAUD4800"),BAUD4800 ));
	baudrate.append(qMakePair( QString("BAUD9600"),BAUD9600 ));
	baudrate.append(qMakePair( QString("BAUD14400             //WINDOWS ONLY"),BAUD14400 ));
	baudrate.append(qMakePair( QString("BAUD19200"),BAUD19200 ));
	baudrate.append(qMakePair( QString("BAUD38400"),BAUD38400 ));
	baudrate.append(qMakePair( QString("BAUD56000             //WINDOWS ONLY"),BAUD56000 ));
	baudrate.append(qMakePair( QString("BAUD57600"),BAUD57600 ));
	baudrate.append(qMakePair( QString("BAUD76800             //POSIX ONLY"),BAUD76800 ));
	baudrate.append(qMakePair( QString("BAUD115200"),BAUD115200 ));
	baudrate.append(qMakePair( QString("BAUD128000            //WINDOWS ONLY"),BAUD128000 ));
	baudrate.append(qMakePair( QString("BAUD256000             //WINDOWS ONLY"),BAUD256000 ));
	for ( int i = 0; i < baudrate.size(); i++)
	{
		baudRateComboBox->addItem(baudrate[i].first);
	}

    databitstype.append(qMakePair(QString("DATA_5"),DATA_5));
   	databitstype.append(qMakePair(QString("DATA_6"),DATA_6));
    databitstype.append(qMakePair(QString("DATA_7"),DATA_7));
    databitstype.append(qMakePair(QString("DATA_8"),DATA_8));
	for ( int i = 0; i < databitstype.size(); i++)
	{
		dataBitsTypeComboBox->addItem(databitstype[i].first);
	}

	paritytype.append(qMakePair(QString("PAR_NONE"),PAR_NONE));
	paritytype.append(qMakePair(QString("PAR_ODD"),PAR_ODD));
	paritytype.append(qMakePair(QString("PAR_EVEN"),PAR_EVEN));
	paritytype.append(qMakePair(QString("PAR_MARK               //WINDOWS ONLY"),PAR_MARK));
	paritytype.append(qMakePair(QString("PAR_SPACE"),PAR_SPACE));
	for ( int i = 0; i < paritytype.size(); i++)
	{
		parityTypeComboBox->addItem(paritytype[i].first);
	}

	stopbitstype.append(qMakePair(QString("STOP_1"),STOP_1));
	stopbitstype.append(qMakePair(QString("STOP_1_5               //WINDOWS ONLY"),STOP_1_5));
	stopbitstype.append(qMakePair(QString("STOP_2"),STOP_2));
	for ( int i = 0; i < stopbitstype.size(); i++ )
	{
		stopBitsTypeComboBox->addItem(stopbitstype[i].first);
	}

	flowtype.append(qMakePair(QString("FLOW_OFF"),FLOW_OFF));
	flowtype.append(qMakePair(QString("FLOW_HARDWARE"),FLOW_HARDWARE));
	flowtype.append(qMakePair(QString("FLOW_XONXOFF"),FLOW_XONXOFF));
	for ( int i = 0; i < flowtype.size(); i++)
	{
		flowTypeComboBox->addItem(flowtype[i].first);
	}
}



void QSerialPortTools::testAllPort()
{
    QString portSerieTmp;
	QString number;
	QStringList portSerieTmpList;
	QextSerialPort *porttmp = new QextSerialPort();
//WINDOWS
#ifdef Q_OS_WIN
    /**
    * Parcours COM101 à COM2 pour trouver le port de + au niveau ouvrable
    */
    for (int i = 101 ; i > 3  ; i-- )
    {
        portSerieTmp.setNum(i);
        if ( i < 10 )
            portSerieTmp = "COM"+portSerieTmp;
        else
            portSerieTmp = "\\\\.\\COM"+portSerieTmp;

        porttmp->setPortName(portSerieTmp);
        if ( porttmp->open(QIODevice::ReadOnly) )
        {
            portSerieTmpList<<portSerieTmp;
            porttmp->close();
        }
    }
//LINUX
#else
    /**
    * Parcours /dev/ttyUSB10 à /dev/ttyUSB0 pour trouver le port de + au niveau ouvrable
    */
    for (int i = 10 ; i > -1  ; i-- )
	{
		number.setNum(i);
		portSerieTmp = "/dev/ttyUSB"+number;
        porttmp->setPortName(portSerieTmp);
        if ( porttmp->open(QIODevice::ReadOnly) )
		{
            portSerieTmpList<<portSerieTmp;
            porttmp->close();
        }
    }
#endif
	portComboBox->clear();
	portComboBox->addItems(portSerieTmpList);
}

void QSerialPortTools::setDefaultSettings()
{
	baudRateComboBox->setCurrentIndex(defaultSettings.baudRate());
	dataBitsTypeComboBox->setCurrentIndex(defaultSettings.dataBits());
	parityTypeComboBox->setCurrentIndex(defaultSettings.parity());
	stopBitsTypeComboBox->setCurrentIndex(defaultSettings.stopBits());
	flowTypeComboBox->setCurrentIndex(defaultSettings.flowControl());
}

void QSerialPortTools::applySettings()
{
	portSettings->setPortName( portComboBox->currentText());
	portSettings->setBaudRate(baudrate[baudRateComboBox->currentIndex()].second);
	portSettings->setDataBits(databitstype[dataBitsTypeComboBox->currentIndex()].second);
	portSettings->setParity(paritytype[parityTypeComboBox->currentIndex()].second);
	portSettings->setStopBits(stopbitstype[stopBitsTypeComboBox->currentIndex()].second);
	portSettings->setFlowControl(flowtype[flowTypeComboBox->currentIndex()].second);
}

void QSerialPortTools::cancel()
{
	QWidget::close();
}

void QSerialPortTools::ok()
{
	applySettings();
	QWidget::close();
}

bool QSerialPortTools::connectPort(QextSerialPort &port,QIODevice::OpenMode mode)
{
    if ( port.portName() == "" )
    {
        return false;
    }

    if ( !port.isOpen()){
        port.setPortName(port.portName());
        port.open( mode );
        port.setParity(port.parity());
        port.setDataBits(port.dataBits());
        port.setStopBits(port.stopBits());
        port.setBaudRate(port.baudRate());
        port.setFlowControl(port.flowControl());
    }

    if ( !port.isOpen())
    {
        port.setPortName(QString(""));
    }

    return port.isOpen();
}

void QSerialPortTools::disconnectPort(QextSerialPort &port)
{
	port.close();
}

bool QSerialPortTools::portReady(QextSerialPort &port)
{
	if ( port.portName() == "" )
		return false;

	if ( port.open(QIODevice::ReadWrite) )
	{
		port.close();
	}
	else
	{
		port.setPortName(QString(""));
		return false;
	}

	return true;
}
